﻿using UnityEngine;
using UnityEngine.UI;
using System.Collections;
using System;
using System.Collections.Generic;

public class AR_TEST : MonoBehaviour {
    public GameObject FileSystemWindow;
    public UI_FileBrowser _uiFB;
    public string myPath;

    public Text fileNameTextBox;
    public Text fileContentsTextBox;
    public Text nodeContentsTextBox;
    public Text edgeContentsTextBox;

    Dictionary<string, int> nodeCount = new Dictionary<string, int>();
    Dictionary<string, int> edgeCount = new Dictionary<string, int>();
    Dictionary<string, int> nodesInFile = new Dictionary<string, int>();

    public Dictionary<string, int> GetNodes() { return nodeCount; }
    public Dictionary<string, int> GetEdges() { return edgeCount; }

    // Use this for initialization
    void Start () {
	
	}
	
	// Update is called once per frame
	void Update () {
	
	}
    public void Button_LoadFile() {
        string fileName = fileNameTextBox.text;
        LoadFile(fileName);
    }
    private void LoadFile(string fileName) { 
        string wholeFileLine;
        wholeFileLine = GetContents(fileName);
        EnsureTextFits(wholeFileLine, fileContentsTextBox);
    }
    public void Button_ParseFile()    {
        string fileName = fileNameTextBox.text;
        ParseFile(fileName);
    }
    private void ParseFile(string fileName) { 
        string wholeFileLine;
        //string[] lines;
        List<Node> nodes = new List<Node>();
        List<Edge> edges = new List<Edge>();

        wholeFileLine = GetContents(fileName);
        EnsureTextFits(wholeFileLine, fileContentsTextBox);
        //lines = GetLines(wholeFileLine);
        nodes = GenerateNodeList(wholeFileLine);
        DisplayNodesInTextBox(nodes, nodeContentsTextBox);
        edges = GenerateEdgeList(nodes);
        DisplayEdgesInTextBox(edges, edgeContentsTextBox);

    }
    public void Button_ParseMultipleFiles() {
        string fileName = fileNameTextBox.text;
        ParseMultipleFiles(fileName);
    }
    private void ParseMultipleFiles(string AllFileNames) {
        //Fills these dictionaries and displays the output
        //Moved to global variables AND will use this to generate heat map data
        //Dictionary<string, int> nodeCount = new Dictionary<string, int>();
        //Dictionary<string, int> edgeCount = new Dictionary<string, int>();
        //Dictionary<string, int> nodesInFile = new Dictionary<string, int>();

        string[] fileNames = AllFileNames.Split(new char[] { '\r', '\n' });
        foreach (string fileName in fileNames) {
            Debug.Log("processing file:" +fileName);
            List<Node> nodes = new List<Node>();
            List<Edge> edges = new List<Edge>();
            string wholeFileLine = GetContents(fileName);
            nodes = GenerateNodeList(wholeFileLine);
            edges = GenerateEdgeList(nodes);
            nodesInFile.Add(fileName, nodes.Count);
            foreach (Node n in nodes) {
                Debug.Log("Processing node: :" + n.ToString());
                string temp = n.ToString();
                int value;
                // use the node.Tostring value as the key (1000 unique combinations)
                // use the value parameter as a count
                // don't use ++ with dictionaries
                // if the key doesn't exist, add it. 
                // http://www.dotnetperls.com/trygetvalue

                if (nodeCount.TryGetValue(temp, out value)) {
                    nodeCount[temp] = value + 1;
                }
                else {
                    nodeCount.Add(temp, 1);
                }
            }
            foreach (Edge e in edges) {
                Debug.Log("Processing edge: " + e.ToString());
                string temp = e.ToString();
                int value;
                if (edgeCount.TryGetValue(temp, out value)) {
                    edgeCount[temp] = value + 1;
                }
                else {
                    edgeCount.Add(temp, 1);
                }
            }
            
        }
        Debug.Log("Displaying Output");
        DisplayCountInTextBox(nodesInFile, fileContentsTextBox);
        DisplayCountInTextBox(nodeCount, nodeContentsTextBox);
        DisplayCountInTextBox(edgeCount, edgeContentsTextBox);
    }

    private string GetContents(string fileName) {
        string line;
        using (System.IO.StreamReader sr = new System.IO.StreamReader(fileName)) {
            line = sr.ReadToEnd();
        }
        //string templine = line.Length > 15000 ? line.Substring(0, 15000) : line;
        //fileContentsTextBox.text = templine;
        return line;
    }
    //I probably should be able to do this with generics
    // but not tonight...
 
    private void DisplayCountInTextBox(Dictionary<string, int> data, Text textbox) {
        System.Text.StringBuilder builder = new System.Text.StringBuilder();
        List<string> temp = new List<string>();
        foreach (var pair in data) {
            temp.Add(pair.Key);
        }
        temp.Sort();
        foreach (string key in temp) {//sorted list
            builder.Append("**" +data[key].ToString() + "** \t" + key + "\n");
        }
        EnsureTextFits(builder.ToString(), textbox);

    }
    private void DisplayEdgesInTextBox(List<Edge> edges, Text textbox) {
        System.Text.StringBuilder builder = new System.Text.StringBuilder();
        foreach (Edge e in edges)
        {
            builder.Append(e.ToString()).Append("\n");
        }
        EnsureTextFits(builder.ToString(), textbox);
    }
    private void DisplayNodesInTextBox(List<Node> nodes, Text textbox) {
           System.Text.StringBuilder builder = new System.Text.StringBuilder();
           foreach (Node n in nodes) {
                builder.Append(n.ToString()).Append("\n");
            }
        EnsureTextFits(builder.ToString(), textbox);
    }


    private List<Edge> GenerateEdgeList(List<Node> nodes) {
        Node rootNode = new Node(3, 11);
        Node previousNode = new Node();
        List<Edge> edges = new List<Edge>();
        foreach (Node n in nodes) {
            //if the current node = the root node, then you are at the start
            if (n.CheckEquals(n, rootNode))
            {
                previousNode = n;
            }
            else {
                Edge edge = new Edge(previousNode, n);
                edges.Add(edge);
                previousNode = n;
            }
        }
        return edges;
    }

    private void EnsureTextFits(string t, Text textbox) {
        textbox.text = t.Length > 15000 ? t.Substring(0, 15000) + "\n\t[file truncated]" : t;
    }

    private List<Node> GenerateNodeList(string wholefile) {
        string strA = "player: Starting conversation with Game Master";
        string strB = "<color=#ff0000ff>";
        string strC = "<color=#0000ffff>";
        List<Node> nodes = new List<Node>();
        //Node currentNode = new Node();
        //Node previousNode = new Node();

        string[] nonsense = GetLines(wholefile);
        foreach (string s in nonsense) {
            //null node ->skip it
            if (s == null) { }
            //starting node ->reset edge list
            else if (s.Contains(strA)) { }
            //conversation nodes node
            else if (s.Contains(strB) || s.Contains(strC)) {
            //parse the current node from the text and add it to the list
                char[] inLineDelimiters = new char[] { '>', ':', ' ' };
                string[] parts = s.Split(inLineDelimiters, System.StringSplitOptions.RemoveEmptyEntries);
                Node _node = new Node();
                bool res1 = int.TryParse(parts[1], out _node.ConvID);
                bool res2 = int.TryParse(parts[2], out _node.NodeID);
                if (res1 && res2) { nodes.Add(_node); }
            }
            // unknown ->skip it
            else { }

        }

        return nodes;
    }

    private string[] GetLines(string line) {
            char[] delimiters =new char[] { '\r', '\n' };
            string[] parts = line.Split(delimiters);
        return parts;
    }


    public void Button_LoadFileWithMenu()
    {
        StartCoroutine(LoadFile_Coroutine()); // Simple call to the coroutine
    }

    IEnumerator LoadFile_Coroutine()
    {
        FileSystemWindow.SetActive(true); // Enable the GameObject
        _uiFB.Open(myPath); // Open the file browser

        while (string.IsNullOrEmpty(_uiFB.GetPath())) // Wait until the file browser return a path (meaning a file has been opened
            yield return null;

        fileNameTextBox.text = _uiFB.GetPath(); // Get the path and display it in your UI.
        FileSystemWindow.SetActive(false); // Disable the GameObject
        _uiFB.ResetPath(); // Added this function because it's necessary, it just reset a variable in the FileBrowser
    }

}





//http://forum.unity3d.com/threads/ui-text-character-limit.359729/
//    2ND LAST ENTRY...
//Just as an FYI, I did add in the latest release of TextMesh Pro a feature where text that exceeds the 65,535 vertices limit is automatically split into sub text objects. From a user's point of view, this is just a single text object which works like any other text object but happens to contain lots of characters.

//I'll try to build an example scene and do some tests.